==> openstory-8.x-2.9.tar.gz: openstory/src/Controller/OsEntityDefinition.php <==
<?php

namespace Drupal\openstory\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\jsonapi\ResourceResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Class OsEntityDefinition.
 *
 * @package Drupal\openstory\Controller
 */
class OsEntityDefinition extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $fieldManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Base fields needed.
   *
   * @var array
   */
  protected $baseFields = [
    'promote' => 'promote',
    'status' => 'status',
    'title' => 'title',
    'sticky' => 'sticky',
    'name' => 'name',
    'mail' => 'mail',
    'pass' => 'pass',
    'roles' => 'roles',
  ];

  /**
   * OsEntityDefinition constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $fieldManager
   *   The entity field manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, EntityFieldManagerInterface $fieldManager, ModuleHandlerInterface $moduleHandler) {
    $this->entityTypeManager = $entityTypeManager;
    $this->fieldManager = $fieldManager;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('module_handler')
    );
  }

  /**
   * Returns information about all entity types on the systems.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   Return the definition via jsonapi.
   */
  public function entityDefinition($entityType, $bundle) {

    if (!empty($entityType) && !empty($bundle)) {
      $definition = $this->getDefinition($entityType, $bundle);
      $entity = $this->entityTypeManager->getDefinition($entityType);
      if ($bundle_entity_type_id = $entity->getBundleEntityType()) {
        $entityBundle = $this->entityTypeManager->getStorage($bundle_entity_type_id)->load($bundle);
        $definition['entityLabel'] = $entityBundle->label();
      }

      // Add base fields to entity definition.
      $definitions = $this->fieldManager->getFieldDefinitions($entityType, $bundle);
      foreach ($definitions as $fieldName => $fieldDefinition) {
        if ($fieldDefinition instanceof BaseFieldDefinition) {
          if (!array_key_exists($fieldName, $this->baseFields)) {
            continue;
          }
          $fieldInfo = [
            'isBaseField' => TRUE,
            'label' => $fieldDefinition->getLabel(),
            'type' => $fieldDefinition->getType(),
            'description' => $fieldDefinition->getDescription(),
            'required' => $fieldDefinition->isRequired(),
            'readonly' => $fieldDefinition->isReadOnly(),
            'cardinality' => $fieldDefinition->getFieldStorageDefinition()
              ->getCardinality(),
            'settings' => $fieldDefinition->getSettings(),
          ];
          $definitions = $fieldDefinition->toArray();
          if (!empty($definitions) && !empty($definitions['default_value'])) {
            $fieldInfo['default_value'] = $definitions['default_value'];
          }
          else {
            // Add for roles, the default values.
            if ($entityType === 'user' && $fieldName === 'roles') {
              $roles = array_map(['\Drupal\Component\Utility\Html', 'escape'], user_role_names(TRUE));
              $fieldInfo['default_value'] = $roles;
            }
          }

          if ($fieldName === 'status') {
            if ($this->moduleHandler->moduleExists('content_moderation')) {
              $fieldInfo['content_moderation'] = TRUE;
            }
            else {
              $fieldInfo['content_moderation'] = FALSE;
            }
          }
          $definition[$fieldName] = $fieldInfo;
        }
      }
      $response = new ResourceResponse($definition, 200);
      return $response;
    }
    else {
      throw new BadRequestHttpException('Parameters entity type and bundle are mandatory.');
    }

  }

  /**
   * Get entity definition.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle.
   *
   * @return array
   *   The definition.
   */
  public function getDefinition($entityType, $bundle) {
    if (!$this->entityTypeManager->hasDefinition($entityType)) {
      throw new NotFoundHttpException($this->t('No entity type found: @type.', ['@type' => $entityType]));
    }
    $definition = [];
    // Create the entity to normalize it via json api.
    $entity = $this->getEntity($entityType, $bundle);
    if (!empty($entity)) {
      // Check if bundle exists in case the entity type is a node.
      $contentEntityType = $this->entityTypeManager->getDefinition($entityType);
      if ($bundleEntityTypeId = $contentEntityType->getBundleEntityType()) {
        $entityBundle = $this->entityTypeManager->getStorage($bundleEntityTypeId)->load($bundle);
        if (!$entityBundle) {
          throw new NotFoundHttpException($this->t('No bundle "@bundle" found for entity type @type',
            ['@type' => $entityType, '@bundle' => $bundle]
          )
          );
        }
      }

      // Get the view display of the content type via json api.
      $entityViewDisplay = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($entity);
      $fields = $this->getEntityFields($entityViewDisplay);
      if (!empty($fields)) {
        foreach ($fields as $key => $value) {
          // Get fieldStorage and normalize it.
          $fieldStorage = $this->entityTypeManager->getStorage('field_storage_config')->load($entityType . '.' . $value);
          if (!empty($fieldStorage)) {
            $fieldStorageNormalized = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($fieldStorage);
            $this->getStorageProperties($fieldStorageNormalized, $definition);
          }

          // Get field config and normalize it.
          $fieldConfig = $this->entityTypeManager->getStorage('field_config')->load($entityType . '.' . $bundle . '.' . $value);
          if (!empty($fieldConfig)) {
            $fieldConfigNormalized = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($fieldConfig);
            $this->getConfigProperties($fieldConfigNormalized, $definition);
          }
          // If the field name exists as property in definition,
          // check if it is a complex field.
          if (!empty($definition) && array_key_exists($value, $definition)) {
            if (!empty($definition[$value]['storage'])) {
              // See if there are nested fields.
              if (!empty($definition[$value]['storage']['type'])) {
                switch ($definition[$value]['storage']['type']) {
                  case 'field_collection':
                    $this->getNestedFields($definition, $value, 'field_collection');
                    break;

                  case 'entity_reference_revisions':
                    $this->getNestedFields($definition, $value, 'entity_reference_revisions');
                    break;

                  case 'datetime':
                    if (!empty($definition[$value]['config'])) {
                      $config = $definition[$value]['config'];
                      $definition[$value]['config']['osTimeZone'] = drupal_get_user_timezone();
                      if (!empty($config['default_value'])) {
                        $newDefaultValue = [];
                        foreach ($config['default_value'] as $def => $defValue) {
                          $temp = [];
                          $temp['default_date_type'] = $defValue['default_date_type'];
                          $temp['default_date'] = strtotime($defValue['default_date']);
                          array_push($newDefaultValue, $temp);
                        }
                        $definition[$value]['config']['default_value'] = $newDefaultValue;
                      }
                    }
                    break;

                  case 'timestamp':
                    $definition[$value]['config']['osTimeZone'] = drupal_get_user_timezone();
                    break;

                  default:
                }
              }
            }
          }
        }
      }
    }

    return $definition;
  }

  /**
   * Get nested fields.
   *
   * @param array $definition
   *   Entity definition.
   * @param string $fieldName
   *   Get the fields of this field.
   * @param string $nestedEntityType
   *   Entity type.
   */
  public function getNestedFields(array &$definition, $fieldName, $nestedEntityType) {
    switch ($nestedEntityType) {
      case 'field_collection':
        $nestedDefinition = $this->getDefinition('field_collection_item', $fieldName);
        $definition[$fieldName]['nested_fields'][] = $nestedDefinition;
        break;

      case 'entity_reference_revisions':
        $this->getEntityNestedFields($definition, $fieldName, 'paragraph');
        break;

      default:
    }
  }

  /**
   * Get the nested fields of a complex field.
   *
   * @param array $definition
   *   Entity definition.
   * @param string $fieldName
   *   Get nested fields of this field.
   * @param string $entity
   *   Entity type.
   */
  public function getEntityNestedFields(array &$definition, $fieldName, $entity) {
    if (!empty($definition[$fieldName]['config']) && !empty($definition[$fieldName]['config']['settings'])) {
      $contents = $definition[$fieldName]['config']['settings']['handler_settings']['target_bundles'];
      if (!empty($contents)) {
        foreach ($contents as $content => $contentValue) {
          $nestedDefinition = $this->getDefinition($entity, $content);
          if (!empty($nestedDefinition)) {
            $definition[$fieldName]['nested_fields'][] = $nestedDefinition;
          }
        }
      }
    }
  }

  /**
   * Get the entity.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   Return the entity.
   */
  public function getEntity($entityType, $bundle) {
    switch ($entityType) {
      case 'node':
        return $this->entityTypeManager->getStorage('node')->create(['type' => $bundle]);

      case 'taxonomy_term':
        return $this->entityTypeManager->getStorage('taxonomy_term')->create(['vid' => $bundle]);

      case 'user':
        return $this->entityTypeManager->getStorage('user')->create([
          'name' => 'serialization_test_user',
          'mail' => 'foo@example.com',
          'pass' => '123456',
        ]);

      case 'comment':
        $values = [];
        $definition = $this->entityTypeManager->getDefinition("comment");
        $bundle_key = $definition->get('entity_keys')['bundle'];
        $values[$bundle_key] = $bundle;
        return $this->entityTypeManager->getStorage('comment')->create($values);

      case 'contact_message':
        return $this->entityTypeManager->getStorage('contact_message')->create([
          'contact_form' => $bundle,
        ]);

      case 'field_collection_item':
        return $this->entityTypeManager->getStorage('field_collection_item')->create([
          'field_name' => $bundle,
        ]);

      case 'paragraph':
        return $this->entityTypeManager->getStorage('paragraph')->create([
          'type' => $bundle,
        ]);

      default:
    }

  }

  /**
   * Get entity fields.
   *
   * @param array $entityViewDisplay
   *   The entity view display.
   *
   * @return array
   *   Return the entity fields.
   */
  public function getEntityFields(array $entityViewDisplay) {
    $fields = [];
    $attributes = $this->checkJsonApiFormat($entityViewDisplay, 'attributes');
    if (!empty($attributes)) {
      foreach ($attributes as $item => $value) {
        // No need base fields.
        if (array_key_exists($item, $this->baseFields)) {
          continue;
        }
        array_push($fields, $item);
      }
    }
    $relationships = $this->checkJsonApiFormat($entityViewDisplay, 'relationships');
    if (!empty($relationships)) {
      foreach ($relationships as $item => $value) {
        // No need base fields.
        if (array_key_exists($item, $this->baseFields)) {
          continue;
        }
        array_push($fields, $item);
      }
    }
    return $fields;
  }

  /**
   * Add to definition properties needed.
   *
   * @param array $fieldStorageNormalized
   *   Normalized field storage via jsonapi.
   * @param array $definition
   *   Variable that hold the entity definition.
   */
  public function getStorageProperties(array $fieldStorageNormalized, array &$definition) {
    $attributes = $this->checkJsonApiFormat($fieldStorageNormalized, 'attributes');
    $definition[$attributes['field_name']]['storage'] = $attributes;
  }

  /**
   * Add to definition properties needed.
   *
   * @param array $fieldConfigNormalized
   *   Normalized field config via jsonapi.
   * @param array $definition
   *   Variable that hold the entity definition.
   */
  public function getConfigProperties(array $fieldConfigNormalized, array &$definition) {
    $attributes = $this->checkJsonApiFormat($fieldConfigNormalized, 'attributes');
    $definition[$attributes['field_name']]['config'] = $attributes;
  }

  /**
   * Check if it is the format expected.
   *
   * @param array $json
   *   The json result from jsonapi.
   * @param string $property
   *   The property needed to check if exists in $json.
   *
   * @return mixed
   *   Return property array.
   */
  public function checkJsonApiFormat(array $json, $property) {
    if (is_array($json)) {
      if (array_key_exists('data', $json)) {
        if (is_array($json['data'])) {
          if (array_key_exists($property, $json['data'])) {
            return $json['data'][$property];
          }
        }
      }
    }
  }

}

==> openstory-8.x-2.12.tar.gz: openstory/src/Controller/OsEntityDefinition.php <==
<?php

namespace Drupal\openstory\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\Entity\BaseFieldOverride;
use Drupal\jsonapi\ResourceResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Class OsEntityDefinition.
 *
 * @package Drupal\openstory\Controller
 */
class OsEntityDefinition extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $fieldManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Base fields needed.
   *
   * @var array
   */
  protected $baseFields = [
    'promote' => 'promote',
    'status' => 'status',
    'title' => 'title',
    'sticky' => 'sticky',
    'name' => 'name',
    'mail' => 'mail',
    'pass' => 'pass',
    'roles' => 'roles',
  ];

  /**
   * OsEntityDefinition constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $fieldManager
   *   The entity field manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, EntityFieldManagerInterface $fieldManager, ModuleHandlerInterface $moduleHandler) {
    $this->entityTypeManager = $entityTypeManager;
    $this->fieldManager = $fieldManager;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('module_handler')
    );
  }

  /**
   * Returns information about all entity types on the systems.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   Return the definition via jsonapi.
   */
  public function entityDefinition($entityType, $bundle) {

    if (!empty($entityType) && !empty($bundle)) {
      $definition = $this->getDefinition($entityType, $bundle);
      $entity = $this->entityTypeManager->getDefinition($entityType);
      if ($bundle_entity_type_id = $entity->getBundleEntityType()) {
        $entityBundle = $this->entityTypeManager->getStorage($bundle_entity_type_id)->load($bundle);
        $definition['entityLabel'] = $entityBundle->label();
      }

      // Add base fields to entity definition.
      $definitions = $this->fieldManager->getFieldDefinitions($entityType, $bundle);
      foreach ($definitions as $fieldName => $fieldDefinition) {
        if ($fieldDefinition instanceof BaseFieldDefinition || $fieldDefinition instanceof BaseFieldOverride) {
          if (!array_key_exists($fieldName, $this->baseFields)) {
            continue;
          }
          $fieldInfo = [
            'isBaseField' => TRUE,
            'label' => $fieldDefinition->getLabel(),
            'type' => $fieldDefinition->getType(),
            'description' => $fieldDefinition->getDescription(),
            'required' => $fieldDefinition->isRequired(),
            'readonly' => $fieldDefinition->isReadOnly(),
            'cardinality' => $fieldDefinition->getFieldStorageDefinition()
              ->getCardinality(),
            'settings' => $fieldDefinition->getSettings(),
          ];
          $definitions = $fieldDefinition->toArray();
          if (!empty($definitions) && !empty($definitions['default_value'])) {
            $fieldInfo['default_value'] = $definitions['default_value'];
          }
          else {
            // Add for roles, the default values.
            if ($entityType === 'user' && $fieldName === 'roles') {
              $roles = array_map(['\Drupal\Component\Utility\Html', 'escape'], user_role_names(TRUE));
              $fieldInfo['default_value'] = $roles;
            }
          }

          if ($fieldName === 'status') {
            if ($this->moduleHandler->moduleExists('content_moderation')) {
              $fieldInfo['content_moderation'] = TRUE;
            }
            else {
              $fieldInfo['content_moderation'] = FALSE;
            }
          }
          $definition[$fieldName] = $fieldInfo;
        }
      }
      $response = new ResourceResponse($definition, 200);
      return $response;
    }
    else {
      throw new BadRequestHttpException('Parameters entity type and bundle are mandatory.');
    }

  }

  /**
   * Get entity definition.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle.
   *
   * @return array
   *   The definition.
   */
  public function getDefinition($entityType, $bundle) {
    if (!$this->entityTypeManager->hasDefinition($entityType)) {
      throw new NotFoundHttpException($this->t('No entity type found: @type.', ['@type' => $entityType]));
    }
    $definition = [];
    // Create the entity to normalize it via json api.
    $entity = $this->getEntity($entityType, $bundle);
    if (!empty($entity)) {
      // Check if bundle exists in case the entity type is a node.
      $contentEntityType = $this->entityTypeManager->getDefinition($entityType);
      if ($bundleEntityTypeId = $contentEntityType->getBundleEntityType()) {
        $entityBundle = $this->entityTypeManager->getStorage($bundleEntityTypeId)->load($bundle);
        if (!$entityBundle) {
          throw new NotFoundHttpException($this->t('No bundle "@bundle" found for entity type @type',
            ['@type' => $entityType, '@bundle' => $bundle]
          )
          );
        }
      }

      // Get the view display of the content type via json api.
      $entityViewDisplay = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($entity);
      $fields = $this->getEntityFields($entityViewDisplay);
      if (!empty($fields)) {
        foreach ($fields as $key => $value) {
          // Get fieldStorage and normalize it.
          $fieldStorage = $this->entityTypeManager->getStorage('field_storage_config')->load($entityType . '.' . $value);
          if (!empty($fieldStorage)) {
            $fieldStorageNormalized = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($fieldStorage);
            $this->getStorageProperties($fieldStorageNormalized, $definition);
          }

          // Get field config and normalize it.
          $fieldConfig = $this->entityTypeManager->getStorage('field_config')->load($entityType . '.' . $bundle . '.' . $value);
          if (!empty($fieldConfig)) {
            $fieldConfigNormalized = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($fieldConfig);
            $this->getConfigProperties($fieldConfigNormalized, $definition);
          }
          // If the field name exists as property in definition,
          // check if it is a complex field.
          if (!empty($definition) && array_key_exists($value, $definition)) {
            if (!empty($definition[$value]['storage'])) {
              // See if there are nested fields.
              if (!empty($definition[$value]['storage']['type'])) {
                switch ($definition[$value]['storage']['type']) {
                  case 'field_collection':
                    $this->getNestedFields($definition, $value, 'field_collection');
                    break;

                  case 'entity_reference_revisions':
                    $this->getNestedFields($definition, $value, 'entity_reference_revisions');
                    break;

                  case 'datetime':
                    if (!empty($definition[$value]['config'])) {
                      $config = $definition[$value]['config'];
                      $definition[$value]['config']['osTimeZone'] = drupal_get_user_timezone();
                      if (!empty($config['default_value'])) {
                        $newDefaultValue = [];
                        foreach ($config['default_value'] as $def => $defValue) {
                          $temp = [];
                          $temp['default_date_type'] = $defValue['default_date_type'];
                          $temp['default_date'] = strtotime($defValue['default_date']);
                          array_push($newDefaultValue, $temp);
                        }
                        $definition[$value]['config']['default_value'] = $newDefaultValue;
                      }
                    }
                    break;

                  case 'timestamp':
                    $definition[$value]['config']['osTimeZone'] = drupal_get_user_timezone();
                    break;

                  default:
                }
              }
            }
          }
        }
      }
    }

    return $definition;
  }

  /**
   * Get nested fields.
   *
   * @param array $definition
   *   Entity definition.
   * @param string $fieldName
   *   Get the fields of this field.
   * @param string $nestedEntityType
   *   Entity type.
   */
  public function getNestedFields(array &$definition, $fieldName, $nestedEntityType) {
    switch ($nestedEntityType) {
      case 'field_collection':
        $nestedDefinition = $this->getDefinition('field_collection_item', $fieldName);
        $definition[$fieldName]['nested_fields'][] = $nestedDefinition;
        break;

      case 'entity_reference_revisions':
        $this->getEntityNestedFields($definition, $fieldName, 'paragraph');
        break;

      default:
    }
  }

  /**
   * Get the nested fields of a complex field.
   *
   * @param array $definition
   *   Entity definition.
   * @param string $fieldName
   *   Get nested fields of this field.
   * @param string $entity
   *   Entity type.
   */
  public function getEntityNestedFields(array &$definition, $fieldName, $entity) {
    if (!empty($definition[$fieldName]['config']) && !empty($definition[$fieldName]['config']['settings'])) {
      $contents = $definition[$fieldName]['config']['settings']['handler_settings']['target_bundles'];
      if (!empty($contents)) {
        foreach ($contents as $content => $contentValue) {
          $nestedDefinition = $this->getDefinition($entity, $content);
          if (!empty($nestedDefinition)) {
            $definition[$fieldName]['nested_fields'][] = $nestedDefinition;
          }
        }
      }
    }
  }

  /**
   * Get the entity.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   Return the entity.
   */
  public function getEntity($entityType, $bundle) {
    switch ($entityType) {
      case 'node':
        return $this->entityTypeManager->getStorage('node')->create(['type' => $bundle]);

      case 'taxonomy_term':
        return $this->entityTypeManager->getStorage('taxonomy_term')->create(['vid' => $bundle]);

      case 'user':
        return $this->entityTypeManager->getStorage('user')->create([
          'name' => 'serialization_test_user',
          'mail' => 'foo@example.com',
          'pass' => '123456',
        ]);

      case 'comment':
        $values = [];
        $definition = $this->entityTypeManager->getDefinition("comment");
        $bundle_key = $definition->get('entity_keys')['bundle'];
        $values[$bundle_key] = $bundle;
        return $this->entityTypeManager->getStorage('comment')->create($values);

      case 'contact_message':
        return $this->entityTypeManager->getStorage('contact_message')->create([
          'contact_form' => $bundle,
        ]);

      case 'field_collection_item':
        return $this->entityTypeManager->getStorage('field_collection_item')->create([
          'field_name' => $bundle,
        ]);

      case 'paragraph':
        return $this->entityTypeManager->getStorage('paragraph')->create([
          'type' => $bundle,
        ]);

      default:
    }

  }

  /**
   * Get entity fields.
   *
   * @param array $entityViewDisplay
   *   The entity view display.
   *
   * @return array
   *   Return the entity fields.
   */
  public function getEntityFields(array $entityViewDisplay) {
    $fields = [];
    $attributes = $this->checkJsonApiFormat($entityViewDisplay, 'attributes');
    if (!empty($attributes)) {
      foreach ($attributes as $item => $value) {
        // No need base fields.
        if (array_key_exists($item, $this->baseFields)) {
          continue;
        }
        array_push($fields, $item);
      }
    }
    $relationships = $this->checkJsonApiFormat($entityViewDisplay, 'relationships');
    if (!empty($relationships)) {
      foreach ($relationships as $item => $value) {
        // No need base fields.
        if (array_key_exists($item, $this->baseFields)) {
          continue;
        }
        array_push($fields, $item);
      }
    }
    return $fields;
  }

  /**
   * Add to definition properties needed.
   *
   * @param array $fieldStorageNormalized
   *   Normalized field storage via jsonapi.
   * @param array $definition
   *   Variable that hold the entity definition.
   */
  public function getStorageProperties(array $fieldStorageNormalized, array &$definition) {
    $attributes = $this->checkJsonApiFormat($fieldStorageNormalized, 'attributes');
    $definition[$attributes['field_name']]['storage'] = $attributes;
  }

  /**
   * Add to definition properties needed.
   *
   * @param array $fieldConfigNormalized
   *   Normalized field config via jsonapi.
   * @param array $definition
   *   Variable that hold the entity definition.
   */
  public function getConfigProperties(array $fieldConfigNormalized, array &$definition) {
    $attributes = $this->checkJsonApiFormat($fieldConfigNormalized, 'attributes');
    $definition[$attributes['field_name']]['config'] = $attributes;
  }

  /**
   * Check if it is the format expected.
   *
   * @param array $json
   *   The json result from jsonapi.
   * @param string $property
   *   The property needed to check if exists in $json.
   *
   * @return mixed
   *   Return property array.
   */
  public function checkJsonApiFormat(array $json, $property) {
    if (is_array($json)) {
      if (array_key_exists('data', $json)) {
        if (is_array($json['data'])) {
          if (array_key_exists($property, $json['data'])) {
            return $json['data'][$property];
          }
        }
      }
    }
  }

}
